package listactors
import scala.actors._
import scala.actors.Actor._


  /**
   * Die Case Classes dienen hier der Nachrichtenbermittlung an den ListCalculations Actor
   * SimpleCalculation, AsyncCalculation und FutureCalculation berbringen jeweils
   * einen MapListProcessor um die Elemente einer Liste auf neue Werte zu mappen.
   * Ob dieses Mappen nebenlufig geschieht hngt vom Case ab (Fallunterscheidung siehe in receive)
   *   
   */

case class SimpleCalculation ( lp : MapListProcessor )
case class AsyncCalculation  ( lp: MapListProcessor , numberOfActors : Int )
case class FutureCalculation ( lp: MapListProcessor , numberOfActors : Int )
case class SetResultList (l : List [Int])


  /**
   *  ListCalculations sind Objekte mit zwei Listen
   *    data: diese Liste enthlt die Ausgangswerte
   *    	  neue Werte werden ber Int - Nachrichten hinzugefgt, wobei
   *          der gesendete Int Wert durch eine modifier Funktion schon in einen
   *          anderern Startwert umgewandelt wird.
   *          Sendet man etwa die Werte 1 , 2 , 3 und nutzt einen Modifier, der jeden
   *          Wert mit 2 multipliziert (x => x *2), dann ergibt dies List ( 6 , 4, 2),
   *          da jeder neu hinzugefgte Wert mit 2 multipliziert und vor die Liste als neuer
   *          Kopf gesetzt wird.
   *          
   *    resultData: dieses Feld soll spter die gemappte Liste enthalten 
   *     
   *          
   */


class ListCalculations ( idName : String , modifier : Int => Int) extends Actor{

   start()

   
   private var data : List [Int] = List()			// wird mit Startwerten gefllt via Int Nachrichten
   private var resultData : List [Int] = List()     // wird mit setResult gesetzt, sobald das Ergebnis vorliegt


   // Implementierung des Actors:
   def act () {
     
     while (true) {
       

       // Empfangen von Int, SimpleCalculation, AsyncCalculation, 
       // FutureCalculation, SetResultList - Nachrichten 
       // TODO: Bitte implementieren Sie die Funktionen, mit denen auf die
       //       AsyncCalculation und FutureCalculation - Nachrichten reagiert wird
       receive {
         case value : Int => data = modifier (value) :: data 
         case sc: SimpleCalculation => setResult ( sc.lp.process(data) ,"SC-"+sc.lp.name)           // ACHTUNG - WAS IST HIER KRITISCH?!?	    
	     case ac: AsyncCalculation => setResult ( List() , "TODO Aufgabe 2: Implement Calculation with async. multiple actors"  )         
	     case fc: FutureCalculation => setResult ( List() , "TODO Aufgabe 2: Implement Calculation with Futures"  )
	     case result:SetResultList => { resultData = result.l ; println ("AC Result:" + resultData )}
         
         
       }
     }
    }   
   
   
  /**
   * Mit dieser Methode wird die Ergebnis-Liste gesetzt sobald diese vorliegt.
   * Als Nebeneffekt werden die ursprngliche und die neue Liste auf der Konsole ausgegeben,
   * damit Sie die Ergebnisse besser berprfen knnen.
   * @param result eine Liste mit neuberechneten (gemappten) Int Werten
   * @param method beschreibender Text - um nachzuvollziehen mit welcher Vorgehensweise das Ergebnis berechnet wurde, nur fr den Seiteneffekt
   */   
   
   private def setResult (result : List[Int] , method : String) {
      resultData = result      
      println ( method + " of " + this )
     
   }
  
   
   override def toString = idName + ": " + data.toString() + " --> " + resultData.toString()
 
   
   
   
}